/* R-Dev-Ino Mapper 
   - uses I2C or Serial communication interfaces
   - uses a Nokia Color LCD
   - I2C master or slave mode
   - uses one Ping sensor
   - uses 2 Sharp Long Range sensors
   - uses 1 Thermopile Array sensor
*/ 


// R-Dev-Ino board pinout:
//
// Ext  RDI Funct Arduino  ATmega168      Arduino Funct RDI External
//                       +-----\/----+
//          Reset       1| PC6   PC5 |28  D19 A5  SCL
//          Rx    D0    2| PD0   PC4 |27  D18 A4  SDA
//          Tx    D1    3| PD1   PC3 |26  D17 A3        
//      Red Int0  D2    4| PD2   PC2 |25  D16 A2        
//      Grn Int1  D3    5| PD3   PC1 |24  D15 A1        
//      Blu       D4    6| PD4   PC0 |23  D14 A0        
//                      7| VCC   GND |22  
//                      8| GND  AREF |21  
//          Xtal        9| PB6  AVCC |20  
//          Xtal       10| PB7   PB5 |19  D13      SCK  SCK
//      Pan OC0B  D5   11| PD5   PB4 |18  D12      MISO 
//      Tlt OC0A  D6   12| PD6   PB3 |17  D11 OC2A MOSI SDA      
//      Sw1       D7   13| PD7   PB2 |16  D10 OC1B      CSE
//      Sw2       D8   14| PB0   PB1 |15  D 9 OC1A      Rst
//                       +-----------+
//

//#include <NokiaLCD.h>
#include <Servo.h>
#include <Wire.h>

// I2C slaves addresses
#define Compass         0x21
#define Speech          50>>1
#define MotorController 60>>1
#define ServoController 70>>1
#define LedBar          80>>1
#define Mapper          90>>1
#define Brain           100>>1
#define Thermo          208>>1

// I2C commands
#define Forward     1 //MotorController
#define Reverse     2 //MotorController
#define Left        3 //MotorController/ServoController
#define Right       4 //MotorController/ServoController
#define StopAll     5 //MotorController
#define GoFwd       6 //MotorController
#define GoBwd       7 //MotorController
#define Err         8 //MotorController

#define Up          1 //ServoController
#define Down        2 //ServoController

#define Swaist     11 //ServoController
#define SleftSh    12 //ServoController
#define SrightSh   13 //ServoController
#define SleftArm   14 //ServoController
#define SrightArm  15 //ServoController
#define Sscan      16 //ServoController
#define SleftHand  17 //ServoController
#define SrightHand 18 //ServoController
#define SpanHead   19 //ServoController
#define StiltHead  20 //ServoController
#define StoreMove  21 //ServoController
#define PickUp     22 //ServoController
#define Release    23 //ServoController
#define Wave       24 //ServoController

#define LCD1       31 //Speech
#define LCD2       32 //Speech
#define Heading    33 //Speech
#define Node       34 //Speech
#define Room       35 //Speech
#define Ping       36 //Speech
#define Sharp      37 //Speech
#define Angle      38 //Speech
#define Compass    39 //Speech
#define Temp       40 //Speech
#define Cluster    41 //Speech
#define FoundFire  42 //Speech
#define NoFire     43 //Speech

#define Scan       61 //Mapper
#define Fire       62 //Mapper
#define Fetch      63 //Mapper
#define Dance      64 //Mapper

#define Done       99 //Mapper

//IN-OUTs
#define PingPin  14     // digital pin 14
#define SrightPin 1     // analog pin 1
#define SleftPin  2     // analog pin 2
#define SscanPin  3     // analog pin 3

// Nokia LCD pins:
#define RedPin    2     // digital pin 2
#define BluePin   3     // digital pin 3
#define GreenPin  4     // digital pin 4
#define Sw2Pin    5     // digital pin 5
#define Sw1Pin    6     // digital pin 6
#define RESET     9
#define CS       10
#define SDA      11
#define CLK      13

//Servos
#define PanPin    7     // digital pin 7
#define TiltPin   8     // digital pin 8

// Nokia LCD Setup settings
// Basic Colors
#define WHITE 0xFFF
#define BLACK 0x000
#define RED 0xF00
#define GREEN 0x0F0
#define BLUE 0x00F
#define CYAN 0x0FF
#define MAGENTA 0xF0F
#define YELLOW 0xFF0
#define BROWN 0xB22
#define ORANGE 0xFA0
#define PINK 0xF6A

//Variables
byte PingDistance = 0;
byte USValue[181];
byte temperature[] = {0,0,0,0,0,0,0,0,0}; // Array to hold temperature data
byte hotObject = 0;  // fire temperature
byte hottest[2]={0,0};
char text [50];
char byteString[4];
byte x=0;
byte y=0;
byte a=0;
int odometer=1;                   // the metric odometer
byte val = 0;
byte headingData[2];
int j, deg, frac;
boolean room=0;

byte I2C[5]={0,0,0,0,0}; 
#define Address   0
#define Command   1
#define Value1    2
#define Value2    3

#define center  100
#define MapXcenter  65
#define MapYcenter  35

#define SR 1 //Sharp Short Range sensor
#define MR 2 //Sharp Medium Range sensor
#define LR 3 //Sharp Long Range sensor

//LCD
//NokiaLCD nokiaLcd;
Servo Pan;


//-----------------------------------------------------------------------------


void setup() { 
  pinMode(CS, OUTPUT);
  digitalWrite(CS, HIGH);
  pinMode(CLK, OUTPUT);
  digitalWrite(CLK, HIGH);
  pinMode(SDA, OUTPUT);
  digitalWrite(SDA, HIGH);
  pinMode(RESET, OUTPUT);
  digitalWrite(RESET, HIGH);

  pinMode(Sw1Pin, INPUT); 
  digitalWrite(Sw1Pin, HIGH);//turn on pull-up resistor
  pinMode(Sw2Pin, INPUT); 
  digitalWrite(Sw2Pin, HIGH);//turn on pull-up resistor
  pinMode(RedPin, OUTPUT); 
  digitalWrite(RedPin, HIGH);
  pinMode(GreenPin, OUTPUT); 
  digitalWrite(GreenPin, HIGH);
  pinMode(BluePin, OUTPUT); 
  digitalWrite(BluePin, HIGH);
  pinMode(PingPin, OUTPUT); 
  digitalWrite(PingPin, LOW);

  Pan.attach(PanPin);
  Pan.write(center); // 0 - 80 - 180
  //SoftwareServo::refresh();

  //nokiaLcd.lcd_init();
  delay(800);
  //strcpy(text,"Ready");
  //nokiaLcd.lcd_draw_text(PINK, BLACK, 50, 115, text);
  delay(100);
  
  Serial.begin (9600);
  Serial.println("start");

  Wire.begin(Mapper);     // join i2c bus
  Wire.onReceive(I2CreceiveEvent); // register event

} 
  //int d=0;

void loop(){
    //digitalWrite(BluePin, LOW);
  Process_commands();
  //SoftwareServo::refresh();
    //digitalWrite(BluePin, HIGH);
  //delay(50);
  //ScanMap();
  //CheckSensors();
  //isFire();
  //delay(5000);
  //FireFight();
  //testSharp();
  //Read_Compass_Sensor();
  //delay(500);
}  


void testSharp(){
  int Lval=0;
  int Cval=0;
  int Rval=0;
  Lval=Read_Sharp_Sensor(LR, SleftPin);
  //Lval=analogRead(SleftPin);
  Cval=Read_Sharp_Sensor(SR, SscanPin);
  Rval=Read_Sharp_Sensor(LR, SrightPin);
  Serial.print("Left: ");
  Serial.print(Lval, DEC);
  Serial.print("  Center: ");
  Serial.print(Cval, DEC);
  Serial.print("   Right: ");
  Serial.println(Rval, DEC);
}

void Process_commands() {
    switch (I2C[Command]) {
      case Fire:
        FireFight();
        I2C[Command]=0;
        break;
      case Scan:
        ScanMap();
        I2C[Command]=0;
        break;
      case Dance:
        Dancing();
        I2C[Command]=0;
        break;
      case Done:
        odometer = 0;
        I2C[Command]=0;
        break;
    }
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void ScanMap()
{
 // nokiaLcd.lcd_clear(BLUE, 0, 0, 131, 131);
    strcpy(text,"o");
    //nokiaLcd.lcd_draw_text(PINK, BLACK, (131-MapXcenter-3), (131-MapYcenter), text);
  for (a=0; a<181; ++a)
  {
    digitalWrite(GreenPin, LOW);
    ScanAndDisplay();
    digitalWrite(GreenPin, HIGH);
  }
  delay(1000);
  //nokiaLcd.lcd_clear(BLUE, 0, 0, 131, 131);
    strcpy(text,"o");
    //nokiaLcd.lcd_draw_text(PINK, BLACK, (131-MapXcenter-3), (131-MapYcenter), text);
  for (a=180; a>0; --a)
  {
    digitalWrite(BluePin, LOW);
    ScanAndDisplay();
    digitalWrite(BluePin, HIGH);
  }
  delay(500);
  Pan.write(center); // 0 - 80 - 180
  //SoftwareServo::refresh();
  //I2C_transmit(ServoController,PanHead,center,0);
  I2C[Command] = 0;
}

void ScanAndDisplay()
{
    Pan.write(a);
    //SoftwareServo::refresh();
    //I2C_transmit(ServoController,PanHead,a,0);
    USValue[a] = Read_Ping_Sensor();
    if (USValue[a]<200)
    { 
        byte pos=57;
        if (USValue[a]<100)
        {
          strcpy(text,"   ");
          //nokiaLcd.lcd_draw_text(CYAN, BLACK, pos, 115, text);
          pos=63;
        }
        itoa(USValue[a], byteString, 10);
        strcpy(text, byteString);
        //nokiaLcd.lcd_draw_text(CYAN, BLACK, pos, 115, text);
      // multiply the angle° of the servo by 0.01745 etc to convert to radians - a is the angle of the servo ( 90° is to twist result sideways for display)
      x=131-(MapXcenter+USValue[a]*(sin((a-90)*0.0174532925)));
      y=131-(MapYcenter+USValue[a]*(cos((a-90)*0.0174532925)));
      //nokiaLcd.LCD_put_pixel(CYAN, x, y);
    }
}  

void Dancing()
{
  for (int i = 0; i < 3; i++) {
    //I2C_transmit(ServoController,Lshoulder,center,0);
    //I2C_transmit(ServoController,Rshoulder,center,1;
    delay(100);
    //I2C_transmit(PanHead,left,1);
    delay(100);
    //I2C_transmit(PanHead,center,0);
    //I2C_transmit(ServoController,Lshoulder,down,0);
    //I2C_transmit(ServoController,Rshoulder,up,1;
    I2C_transmit(MotorController,Left,0,200);
    delay(1000);
    I2C_transmit(MotorController,StopAll,0,0);
    delay(100);
    //I2C_transmit(PanHead,right,1);
    delay(100);
    //I2C_transmit(PanHead,center,0);
    //I2C_transmit(ServoController,Lshoulder,center,0);
    //I2C_transmit(ServoController,Rshoulder,center,1;
    I2C_transmit(MotorController,Right,0,200);
    delay(1000);
    I2C_transmit(MotorController,StopAll,0,0);
    delay(100);
    //I2C_transmit(PanHead,right,0);
    delay(10);
    I2C_transmit(MotorController,Right,0,200);
    //Lshoulder.write(up);
    //Rshoulder.write(down);
    I2C_transmit(MotorController,StopAll,0,0);
    delay(10);
    //Pan.write(center);
    delay(10);
    I2C_transmit(MotorController,Left,0,200);
    //Lshoulder.write(center);
    //Rshoulder.write(center);
    I2C_transmit(MotorController,StopAll,0,0);
    //Lshoulder.write(down);
    //Rshoulder.write(down);
    I2C_transmit(MotorController,Forward,0,200);
    I2C_transmit(MotorController,StopAll,0,0);
  }
}

byte Find_Hot_Object() {
  Read_Thermopile_Array();
  byte cluster = 0;
  //byte hotObject = temperature[0];  // ambient temperature
  for(byte i = 1; i < 9; i++){  
    if (temperature[i] > hotObject) {
      hotObject = temperature[i]; // store the highest temperature
      cluster = i; // store the location of the hottest spot
    }
  }
  //Serial.print("Hot Object: ");
  //Serial.print(hotObject, DEC);
  //Serial.print("  ");
  //Serial.print("in cluster ");
  //Serial.print(cluster, DEC);
  //Serial.println("   ");
  if (hotObject>28) return cluster; else return 0;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++



//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void I2CreceiveEvent(int i)
{
    digitalWrite(RedPin, LOW);
    //Serial.println("I2C communication received:");
    while(0 < Wire.available()) // loop through all
    {
      I2C[i] = Wire.receive(); // receive bytes
      //Serial.print(i, DEC);
      //Serial.print("  ");
      //Serial.println(I2C[i], DEC);
      i = i-1;
    }
    digitalWrite(RedPin, HIGH);
    if (I2C[Command]==Done) odometer=0;
}

void I2C_transmit(byte Addr, byte Cmnd, byte Val1, byte Val2){
  Wire.beginTransmission(Addr); // transmit to slave device
  Wire.send(Val2);         // sends one byte  
  Wire.send(Val1);         // sends one byte  
  Wire.send(Cmnd);        // sends one byte
  Wire.endTransmission();    // stop transmitting
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

